USE AdventureWorks2012;
GO

-- what happens when you use unsargeable search criteria?

-- check these in Plan Explorer:

SELECT COUNT(*) FROM Sales.SalesOrderHeader 
WHERE YEAR(OrderDate) = 2005 
AND MONTH(OrderDate) = 7;

SELECT COUNT(*) FROM Sales.SalesOrderHeader 
WHERE OrderDate >= '20050701'
  AND OrderDate < '20050801';

-- ALWAYS use an open-ended range

-- what happens when you try to find the "end" of a period?

DECLARE @start DATETIME, @end DATETIME;

SELECT 
  @start = DATEADD(DAY, DATEDIFF(DAY, 1, CURRENT_TIMESTAMP), 0),
  @end   = DATEADD(MILLISECOND, -3, DATEADD(DAY, 1, @start));

SELECT @start, [Datetime] = @end;

SELECT @start, [Smalldatetime] = CONVERT(SMALLDATETIME, @end);

SELECT @start, [Date] = CONVERT(DATE, @end);

SELECT @start, [Datetime2] = CONVERT(DATETIME2(7), @end);


-- Always use open-ended range.
-- Instead of:

SELECT ... WHERE col BETWEEN @start AND @end;

-- Use:

SELECT ... WHERE col >= @start AND col < DATEADD(DAY, 1, @end);


GO
-- NEVER use shorthand
-- POP QUIZ: what will this yield?

DECLARE @d DATE = '20121225';

SELECT 
  DATEPART(w, @d), 
  DATEPART(y, @d), 
  DATEADD(w, 1, @d),
  DATEADD(y, 1, @d);

/*
  (a) 52, 2012, 2013-01-01, 2013-12-25
  (b) 3,  2012, 2013-01-01, 2012-12-26
  (c) 3,  360,  2012-12-26, 2012-12-26
  (d) 52, 12,   2013-01-01, 2013-12-25
*/

